from rich.console import Console
from rich.prompt import Prompt
from rich.status import Status
from rich.panel import Panel


class Style:
    RED = "#cc3333"
    GREEN = "#00a693"
    GRAY = "#6c757d"
    PASSWD = f"bold {GREEN}"

    def __getattr__(self, name):
        def _get_md(text: str) -> str:
            color = getattr(self, name.upper())
            return self.get_md(color, text)

        return _get_md

    def get_md(self, color, text):
        return f"[{color}]{text}[/{color}]"  # [color]text[/color]


class ConsoleStream:
    def __init__(self) -> None:
        self.writer = Console()
        self.styles = Style()

    def panel(self, title, text, **kwargs):
        panel = Panel.fit(text, title=title, **kwargs)
        self.write(panel, justify="left")

    def ask(self, prompt, **kwargs):
        style = kwargs.pop("style", None)
        if style is not None:  # might be needed in the future
            prompt = self.styles.get_md(style, prompt)
        result = ""
        while not result:
            result = Prompt.ask(prompt, console=self.writer, **kwargs)
            if kwargs.get("default", None) is not None:
                break
        return result

    def write(self, text, **kwargs):
        self.writer.print(text, **kwargs)

    def status(self, status, **kwargs):
        return Status(status, console=self.writer, **kwargs)



from allsafe.modules import ConsoleStream, encrypt


__version__ = "1.4.3"

def handle_inputs(console: ConsoleStream):
    addr_sample = console.styles.gray("(e.g Battle.net)")
    addr = console.ask(f"Enter app address/name {addr_sample}")

    username_sample = console.styles.gray("(e.g user123)")
    username = console.ask(f"Enter username {username_sample}")

    case_note = console.styles.gray("(case-sensitive)")
    note = "(do [bold]NOT[/bold] forget this), " + case_note
    secret_key = console.ask(f"Enter secret key {note}")

    return (secret_key, addr, username)

def print_passwds(console: ConsoleStream, passwds: list):
    md_passwds = [console.styles.passwd(i) for i in passwds]
    console.write(
        "\n"
        f"🔒 8-Length Password:\t{md_passwds[0]}\n"
        f"🔏 16-Length Password:\t{md_passwds[1]}\n"
        f"🔐 24-Length Password:\t{md_passwds[2]}\n"
    )

def generate_custom_password(console: ConsoleStream, *args):
    length_note = console.styles.gray("(between 1-64)")
    length = 0
    while not 0 < length < 65:
        answer = console.ask(f"Enter the length {length_note}")
        if answer.isdigit():
            length = int(answer)

    chars_note = console.styles.gray("(enter for default)")
    chars = console.ask(f"Enter password characters {chars_note}",
                        default="", show_default=False)

    passwd_list = encrypt(*args, lengths=(length,), passwd_chars=chars)
    passwd = passwd_list[0]
    console.write(f"\n✅ Here you go: {console.styles.passwd(passwd)}")

def main():
    console = ConsoleStream()
    description = (
        "Get unique password for every app. No need to remeber all of them.\n"
        "No data stored and no internet needed. Use it before every sign-in."
    )
    console.panel("[bold]AllSafe[/bold] Modern Password Generator",
                  description, style=console.styles.GRAY)
    console.write(":link: Github: https://github.com/emargi/allsafe")
    console.write(":gear: Version: " + __version__ + "\n")

    inputs = handle_inputs(console)
    # I don't know if we would ever need statuses
    with console.status("Encrypting..."):
        passwds = encrypt(*inputs)

    print_passwds(console, passwds)
    want_custom_passwd = console.ask(
        "Do you want custom length password?",
        choices=['y', 'n'],
        default='n',
        show_default=False,
        case_sensitive=False,
    )
    if want_custom_passwd == 'n':
        return

    generate_custom_password(console, *inputs)


def run():
    try:
        main()
    except KeyboardInterrupt:
        pass


if __name__ == "__main__":
    run()



from string import ascii_letters, punctuation, digits
from itertools import cycle

import hashlib


PASSWORD_CHARACTERS = digits + ascii_letters + punctuation

def sort_chars(*args) -> list[str]:
    sorted_chars = []
    for arg in args:
        sorted_chars.extend(list(arg))
    sorted_chars.sort()
    return sorted_chars

def get_ords(chars: list) -> list[int]:
    return [ord(char) for char in chars]

def add_ords(ords1: list[int], ords2: list[int]) -> list[int]:
    """This function will sum numbers of two ord lists, pairwise"""
    if len(ords1) > len(ords2):
        larger_ords, shorter_ords = ords1, cycle(ords2)
    else:
        larger_ords, shorter_ords = ords2, cycle(ords1)

    result = []
    for i in larger_ords:
        result.append(i + next(shorter_ords))

    return result

def get_chars(ords: list) -> list[str]:
    return [chr(i) for i in ords]

def calculate_sha256(text: str) -> str:
    return hashlib.sha256(text.encode()).hexdigest()

def _get_steps_based_on_length(cipher_len, passwd_len) -> int:
    return cipher_len // passwd_len

def _convert_hex_to_list_of_ints(hex_string: str, length: int) -> list[int]:
    """
    This function will take a hexadecimal number (`hex_string`) that will be used to generate
    numbers as many as specified in `length` (length of the result list) parameter.
    """
    nums = []
    cipher_len = len(hex_string)
    steps = _get_steps_based_on_length(cipher_len, length)
    for i in range(0, cipher_len, steps):
        nums.append(int(hex_string[i::2], base=16))
    # `hex_string` might not be divisible by `length`, and
    # that results in longer `nums` than the given `length`
    # this is a compatible option, for now.
    return nums[:length]

def turn_into_passwd(hex_string: str, length: int, passwd_chars: str) -> str:
    nums = _convert_hex_to_list_of_ints(hex_string, length)
    new_string = ""
    n_chars = len(passwd_chars)
    for num in nums:
        new_string += passwd_chars[num%n_chars]
    return new_string

def get_unique_and_sorted_chars(chars: str) -> str:
    """Remove duplicate characters, sort them, and return the result."""
    return "".join(sorted(set(chars)))

def encrypt(
        key: str, *args: str,
        lengths:tuple[int]=(8, 16, 24,),
        passwd_chars:str="",
    ) -> list[str]:
    """
    Encrypt texts with a key as following steps:
    - First, unicode of every single character in texts will be sorted
      and stored in a list object.
    - Then the key's unicodes will be stored in another list object.
    - Then unicodes of each list object will be summed pairwise and
      added to a new list.
    - Then the new list will be turned into a list of characters with
      assuming each item of a list (which should be an integer) as a
      unicode.
    - The said characters will be hashed with a specific algorithm.
    - The hashed data (which is a big hexadecimal number) will be
      replaced with other password-safe characters via a complex
      algorithm.
    - The final string is the result which will always be the same
      with the same given data.
    
    set password lengths:
    >>> encrypt(key, *info, lengths=(8, 16, 24))

    set password characters:
    >>> encrypt(key, *info, passwd_chars="abc123")
    """
    extra_strings = (arg.lower() for arg in args)
    char_list = sort_chars(*extra_strings)
    char_ords = get_ords(char_list)
    key_ords = get_ords(key)

    new_ords = add_ords(char_ords, key_ords)
    chars = get_chars(new_ords)
    text = "".join(chars)
    hashed_text = calculate_sha256(text)
    if not passwd_chars:
        # default characters should not be sorted, at least
        # in v1, since that will cause incompatibility
        chars = PASSWORD_CHARACTERS
    else:
        chars = get_unique_and_sorted_chars(passwd_chars)

    passwds = []
    for length in lengths:
        passwds.append(
            turn_into_passwd(hashed_text, length, chars)
        )

    return passwds



from setuptools import setup, find_packages


__version__ = "1.4.3"

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name="AllSafe",
    version=__version__,
    packages=find_packages(),
    install_requires=[
        "rich",
    ],
    entry_points={
        "console_scripts": [
            "allsafe=allsafe.main:run"
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Environment :: Console",
        "Topic :: Security",
    ],
    description="AllSafe, A Modern Password Generator",
    author="Mohamad Reza",
    url="https://github.com/emargi/AllSafe",
    long_description=long_description,
    long_description_content_type="text/markdown",
    project_urls={
        "Source Code": "https://github.com/emargi/Allsafe#readme",
        "Bug Tracker": "https://github.com/emargi/AllSafe/issues",
    },
    keywords="password password-generator tool allsafe generator",
    include_package_data=True,
)
